package xyz.veronie.bgg.ui.parts;

import java.util.Arrays;
import java.util.List;

import javax.annotation.PostConstruct;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

import xyz.veronie.bgg.ui.contributions.SourceFilterBggUser;
import xyz.veronie.bgg.ui.contributions.SourceFilterGeeklist;

/// Has controls to download a list of thing IDs (+some meta info) from BGG.
/// There are different ways to configure which IDs to retrieve.
public class LoadFromBggPart {
	private Composite parent;
	private Composite filterComposite; //< current composite that is shown in filter area
	
	/// show different filter controls depending on selection in cbSource ComboViewer
	public void showFilter(IStructuredSelection selection) {	
		String elem = (String) selection.getFirstElement();
		
		// we already have the selected filter area in place, do nothing
		if(filterComposite != null && filterComposite.toString() == elem) {
			return; 
		}

		// if we need a different filter area, dispose the old one before creating a new one
		if(filterComposite != null) { 
			filterComposite.dispose();
		}

		// now that we know the composite is either null or a different one from the one already shown,
		// and the old one was disposed above, create the new one:
		if(elem == "from BGG user") {
			System.out.println("construct " + elem);
			filterComposite = new SourceFilterBggUser(parent, SWT.FILL);
			filterComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
			parent.requestLayout();
		} else if (elem == "from geeklist") {
			System.out.println("construct " + elem);
			filterComposite = new SourceFilterGeeklist(parent, SWT.FILL);
			filterComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
			parent.requestLayout();
		} else {
			System.out.println("construct " + elem + " (not implemented yet, try another one)");
		}

	}
	
	
	public static Display getDisplay() {
 	   Display display = Display.getCurrent();
 	   // may be null if outside the UI thread
	   if (display == null)
	      display = Display.getDefault();
	   return display;		
    }
	

	@PostConstruct
	public void createControls(Composite parent) {
		this.parent = parent;
		
		// TODO: where do I define this properly? child setting the layout of its parent doesn't feel right
		GridLayout layout = new GridLayout(2, false);
		parent.setLayout(layout);
		
		
		// contains configuration options for download of the bgg thing ID list
		Group dlConfigGroup = new Group(parent, SWT.SHADOW_ETCHED_IN);
		GridData configGrid = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
		dlConfigGroup.setText("Configure download...");
		dlConfigGroup.setLayoutData(configGrid);
		
		GridLayout layoutG = new GridLayout(2, false);
		dlConfigGroup.setLayout(layoutG);
		
		Label lblSource = new Label(dlConfigGroup, SWT.LEFT);
		lblSource.setText("Source: ");
		lblSource.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		
		// choose the bgg data source
		ComboViewer cbSource = new ComboViewer(dlConfigGroup, SWT.READ_ONLY);
		cbSource.setContentProvider(ArrayContentProvider.getInstance());

		List<String> sources = Arrays.asList(new String[] { "from BGG user", "from geeklist", "from family", "by rank", "by year", "by age", "search"});
		cbSource.setInput(sources);
		// TODO: implement all the sources

		cbSource.setSelection(new StructuredSelection(sources.get(0)));
		cbSource.addSelectionChangedListener(new ISelectionChangedListener() {
		    @Override
		    public void selectionChanged(SelectionChangedEvent event) {
		        IStructuredSelection selection = (IStructuredSelection) event
		            .getSelection();
		        if (selection.size() > 0) {
			        showFilter(selection);
		        }
		    }

		});

		
		// choose the bgg sub-site
		Label lblTopic = new Label(dlConfigGroup, SWT.LEFT);
		lblTopic.setText("Subpage: ");
		lblTopic.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		
		ComboViewer cbTopic = new ComboViewer(dlConfigGroup, SWT.READ_ONLY);
		cbTopic.setContentProvider(ArrayContentProvider.getInstance());
		List<String> sites = Arrays.asList(new String[] { "Boardgames", "RPGs", "RPG Items", "Videogames", "Accessories" });
		cbTopic.setInput(sites);
		cbTopic.setSelection(new StructuredSelection(sites.get(0)));

		
		// choose action on result list
		Label lblAct = new Label(dlConfigGroup, SWT.LEFT);
		lblAct.setText("Action on result: ");
		lblAct.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
		
		ComboViewer cbAct = new ComboViewer(dlConfigGroup, SWT.READ_ONLY);
		cbAct.setContentProvider(ArrayContentProvider.getInstance());
		List<String> actions = Arrays.asList(new String[] { "add", "replace", "subtract", "AND", "Mis"});
		cbAct.setInput(actions);
		cbAct.setSelection(new StructuredSelection(actions.get(0)));
		
		
		// prepare area to display filter controls in
//		filterArea = new Composite(parent, SWT.FILL);
//		filterArea.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		showFilter((IStructuredSelection)cbSource.getSelection()); // set content to initial selection
		
		
		// Start retrieval when download button is pushed
		Button btDownload = new Button(parent, SWT.PUSH);
		// TODO: how do I locate icons?
		try {
			Image dlIcon = new Image(getDisplay(), 
				getClass().getResourceAsStream("C:\\Users/goya/workspace/icons/icons/material_design/png/download-button.png"));
			btDownload.setImage(dlIcon);
		} 
		catch(Exception ex) {
			btDownload.setText("Download now");
		}
		btDownload.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
		        System.out.println("Called!");
		    }
		});
		btDownload.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		

		
		
	}
	

	// DISPOSE images!
}
